package com.wyattjmiller.studyhelper

import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.google.android.material.snackbar.Snackbar

open class QuestionActivity : AppCompatActivity() {
    private lateinit var mStudyDb: StudyDatabase
    private lateinit var mSubject: String
    private lateinit var mQuestionList: MutableList<Question>
    private lateinit var mAnswerLabel: TextView
    private lateinit var mAnswerText: TextView
    private lateinit var mAnswerButton: Button
    private lateinit var mQuestionText: TextView
    private lateinit var mShowQuestionsLayout: ViewGroup
    private lateinit var mNoQuestionsLayout: ViewGroup
    private lateinit var mDeletedQuestion: Question

    private var mCurrentQuestionIndex = 0
    private val REQUEST_CODE_NEW_QUESTION = 0
    private val REQUEST_CODE_UPDATE_QUESTION = 1

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_question)

        // Hosting activity provides the subject of the questions to display
        val intent = intent
        mSubject = intent.getStringExtra(EXTRA_SUBJECT)

        // Load all questions for this subject
        mStudyDb = StudyDatabase.getInstance(applicationContext)
        mQuestionList = mStudyDb.getQuestions(mSubject)
        mQuestionText = findViewById(R.id.questionText)
        mAnswerLabel = findViewById(R.id.answerLabel)
        mAnswerText = findViewById(R.id.answerText)
        mAnswerButton = findViewById(R.id.answerButton)
        mShowQuestionsLayout = findViewById(R.id.showQuestionsLayout)
        mNoQuestionsLayout = findViewById(R.id.noQuestionsLayout)

        // Show first question
        showQuestion(0)
    }

    override fun onStart() {
        super.onStart()

        // Are there questions to display?
        if (mQuestionList!!.size == 0) {
            updateAppBarTitle()
            displayQuestion(false)
        } else {
            displayQuestion(true)
            toggleAnswerVisibility()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {

        // Inflate menu for the app bar
        val inflater = menuInflater
        inflater.inflate(R.menu.question_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Determine which app bar item was chosen
        return when (item.itemId) {
            R.id.previous -> {
                showQuestion(mCurrentQuestionIndex - 1)
                true
            }
            R.id.next -> {
                showQuestion(mCurrentQuestionIndex + 1)
                true
            }
            R.id.add -> {
                addQuestion()
                true
            }
            R.id.edit -> {
                editQuestion()
                true
            }
            R.id.delete -> {
                deleteQuestion()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    fun addQuestionButtonClick(view: View?) {
        addQuestion()
    }

    fun answerButtonClick(view: View?) {
        toggleAnswerVisibility()
    }

    private fun displayQuestion(display: Boolean) {

        // Show or hide the appropriate screen
        if (display) {
            mShowQuestionsLayout!!.visibility = View.VISIBLE
            mNoQuestionsLayout!!.visibility = View.GONE
        } else {
            mShowQuestionsLayout!!.visibility = View.GONE
            mNoQuestionsLayout!!.visibility = View.VISIBLE
        }
    }

    private fun updateAppBarTitle() {

        // Display subject and number of questions in app bar
        val actionBar = supportActionBar
        if (actionBar != null) {
            val title = resources.getString(
                R.string.question_number,
                mSubject, mCurrentQuestionIndex + 1, mQuestionList!!.size
            )
            setTitle(title)
        }
    }

    private fun addQuestion() {
        val intent = Intent(this, QuestionEditActivity::class.java)
        intent.putExtra(QuestionEditActivity.EXTRA_SUBJECT, mSubject)
        startActivityForResult(intent, REQUEST_CODE_NEW_QUESTION)
    }

    private fun editQuestion() {
        if (mCurrentQuestionIndex >= 0) {
            val intent = Intent(this, QuestionEditActivity::class.java)
            intent.putExtra(Intent.EXTRA_SUBJECT, mSubject)
            val questionId = mQuestionList[mCurrentQuestionIndex].id
            intent.putExtra(QuestionEditActivity.EXTRA_QUESTION_ID, questionId)
            startActivityForResult(intent, REQUEST_CODE_UPDATE_QUESTION)
        }
    }

    private fun deleteQuestion() {
        mDeletedQuestion = mQuestionList[mCurrentQuestionIndex]
        mStudyDb.deleteQuestion(mDeletedQuestion.id)

        if (mCurrentQuestionIndex >= 0) {
            val questionId = mQuestionList[mCurrentQuestionIndex].id
            mStudyDb.deleteQuestion(questionId)
            mQuestionList.removeAt(mCurrentQuestionIndex)
            if (mQuestionList.size == 0) {
                // No questions left to show
                mCurrentQuestionIndex = -1
                updateAppBarTitle()
                displayQuestion(false)
            } else {
                showQuestion(mCurrentQuestionIndex)
            }

            val snackbar = Snackbar.make(findViewById(R.id.coordinatorLayout), R.string.question_deleted, Snackbar.LENGTH_LONG)
            snackbar.setAction(R.string.undo) {
                mStudyDb.addQuestion(mDeletedQuestion)
                mQuestionList.add(mDeletedQuestion)
                showQuestion(mQuestionList.size - 1)
                displayQuestion(true)
            }
            snackbar.show()
        }
    }


    private fun showQuestion(questionIndex: Int) {

        // Show question at the given index
        var questionIndex = questionIndex
        if (mQuestionList!!.size > 0) {
            if (questionIndex < 0) {
                questionIndex = mQuestionList!!.size - 1
            } else if (questionIndex >= mQuestionList!!.size) {
                questionIndex = 0
            }
            mCurrentQuestionIndex = questionIndex
            updateAppBarTitle()
            val question = mQuestionList!![mCurrentQuestionIndex]
            mQuestionText!!.text = question.text
            mAnswerText!!.text = question.answer
        } else {
            // No questions yet
            mCurrentQuestionIndex = -1
        }
    }

    private fun toggleAnswerVisibility() {
        if (mAnswerText!!.visibility == View.VISIBLE) {
            mAnswerButton!!.setText(R.string.show_answer)
            mAnswerText!!.visibility = View.INVISIBLE
            mAnswerLabel!!.visibility = View.INVISIBLE
        } else {
            mAnswerButton!!.setText(R.string.hide_answer)
            mAnswerText!!.visibility = View.VISIBLE
            mAnswerLabel!!.visibility = View.VISIBLE
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == RESULT_OK && requestCode == REQUEST_CODE_NEW_QUESTION) {
            // Get added question
            val questionId = data!!.getLongExtra(QuestionEditActivity.EXTRA_QUESTION_ID, -1)
            val newQuestion = mStudyDb.getQuestion(questionId)

            // Add newly created question to the question list and show it
            mQuestionList.add(newQuestion)
            showQuestion(mQuestionList.size - 1)
            Toast.makeText(this, R.string.question_added, Toast.LENGTH_SHORT).show()
        } else if (resultCode == RESULT_OK && requestCode == REQUEST_CODE_UPDATE_QUESTION) {
            // Get updated question
            val questionId = data!!.getLongExtra(QuestionEditActivity.EXTRA_QUESTION_ID, -1)
            val updatedQuestion = mStudyDb.getQuestion(questionId)

            // Replace current question in question list with updated question
            val currentQuestion = mQuestionList[mCurrentQuestionIndex]
            currentQuestion.text = updatedQuestion.text
            currentQuestion.answer = updatedQuestion.answer
            showQuestion(mCurrentQuestionIndex)
            Toast.makeText(this, R.string.question_updated, Toast.LENGTH_SHORT).show()
        }
    }

    companion object {
        const val EXTRA_SUBJECT = "com.wyattjmiller.studyhelper.subject"
    }
}
