package com.wyattjmiller.studyhelper

import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Color
import android.os.Bundle
import android.preference.PreferenceManager
import android.view.*
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView

class SubjectActivity : AppCompatActivity(), SubjectDialogFragment.OnSubjectEnteredListener {
    private lateinit var mStudyDb: StudyDatabase
    private lateinit var mSubjectAdapter: SubjectAdapter
    private lateinit var mRecyclerView: RecyclerView
    private lateinit var mSubjectColors: IntArray
    private lateinit var mSelectedSubject: Subject
    private lateinit var mActionMode: ActionMode
    private lateinit var mSharedPrefs: SharedPreferences

    private var mDarkTheme = false
    private var mSelectedSubjectPosition = RecyclerView.NO_POSITION

    override fun onCreate(savedInstanceState: Bundle?) {
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
        mDarkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false);
        if (mDarkTheme) {
            setTheme(R.style.DarkTheme);
        }

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_subject)
        mSubjectColors = resources.getIntArray(R.array.subjectColors)

        // Singleton
        mStudyDb = StudyDatabase.getInstance(applicationContext)
        mRecyclerView = findViewById(R.id.subjectRecyclerView)

        // Create 2 grid layout columns
        val gridLayoutManager: RecyclerView.LayoutManager = GridLayoutManager(applicationContext, 2)
        mRecyclerView.setLayoutManager(gridLayoutManager)

        // Shows the available subjects
        mSubjectAdapter = SubjectAdapter(loadSubjects())
        mRecyclerView.setAdapter(mSubjectAdapter)
    }


    override fun onResume() {
        super.onResume()

        // If theme changed, recreate the activity so theme is applied
        val darkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false)
        if (darkTheme != mDarkTheme) {
            recreate()
        }

        // Load subjects here in case settings changed
        mSubjectAdapter = SubjectAdapter(loadSubjects())
        mRecyclerView.adapter = mSubjectAdapter
    }

    private fun loadSubjects(): MutableList<Subject> {
        val order = mSharedPrefs!!.getString(SettingsFragment.PREFERENCE_SUBJECT_ORDER, "1")
        return when (order!!.toInt()) {
            0 -> mStudyDb.getSubjects(StudyDatabase.SubjectSortOrder.ALPHABETIC)
            1 -> mStudyDb.getSubjects(StudyDatabase.SubjectSortOrder.UPDATE_DESC)
            else -> mStudyDb.getSubjects(StudyDatabase.SubjectSortOrder.UPDATE_ASC)
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.subject_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle item selection
        return when (item.itemId) {
            R.id.settings -> {
                val intent = Intent(this@SubjectActivity, SettingsActivity::class.java)
                startActivity(intent)
                true
            }
            R.id.import_questions -> {
                val intent = Intent(this@SubjectActivity, ImportActivity::class.java)
                startActivity(intent)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onSubjectEntered(subject: String) {
        // Returns subject entered in the SubjectDialogFragment dialog
        if (subject.isNotEmpty()) {
            val sub = Subject(subject)
            if (mStudyDb!!.addSubject(sub)) {
                mSubjectAdapter.addSubject(sub)
                Toast.makeText(this, "Added $subject", Toast.LENGTH_SHORT).show()
            } else {
                val message = resources.getString(R.string.subject_exists, subject)
                Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
            }
        }
    }

    fun addSubjectClick(view: View?) {
        // Prompt user to type new subject
        val manager = supportFragmentManager
        val dialog = SubjectDialogFragment()
        dialog.show(manager, "subjectDialog")
    }

    private inner class SubjectHolder(inflater: LayoutInflater, parent: ViewGroup?) : RecyclerView.ViewHolder(inflater.inflate(R.layout.recycler_view_items, parent, false)), View.OnClickListener, View.OnLongClickListener {
        private var mSubject: Subject? = null
        private val mTextView: TextView

        fun bind(subject: Subject, position: Int) {
            mSubject = subject
            mTextView.text = subject.text
            if (mSelectedSubjectPosition == position) {
                // Make selected subject stand out
                mTextView.setBackgroundColor(Color.RED)
            } else {
                // Make the background color dependent on the length of the subject string
                val colorIndex = subject.text!!.length % mSubjectColors.size
                mTextView.setBackgroundColor(mSubjectColors[colorIndex])
            }
        }
        override fun onClick(view: View) {
            // Start QuestionActivity, indicating what subject was clicked
            val intent = Intent(this@SubjectActivity, QuestionActivity::class.java)
            intent.putExtra(QuestionActivity.EXTRA_SUBJECT, mSubject!!.text)
            startActivity(intent)
        }

        override fun onLongClick(view: View?): Boolean {
            if (mActionMode != null) {
                return false
            }

            mSelectedSubject = mSubject!!
            mSelectedSubjectPosition = adapterPosition

            // Re-bind the selected item
            mSubjectAdapter.notifyItemChanged(mSelectedSubjectPosition)

            // Show the CAB
            mActionMode = this@SubjectActivity.startActionMode(mActionModeCallback)!!
            return true
        }

        private val mActionModeCallback: ActionMode.Callback = object : ActionMode.Callback {
            override fun onCreateActionMode(mode: ActionMode, menu: Menu?): Boolean {
                // Provide context menu for CAB
                val inflater = mode.menuInflater
                inflater.inflate(R.menu.context_menu, menu)
                return true
            }

            override fun onPrepareActionMode(mode: ActionMode?, menu: Menu?): Boolean {
                return false
            }

            override fun onActionItemClicked(mode: ActionMode, item: MenuItem): Boolean {
                // Process action item selection
                return when (item.getItemId()) {
                    R.id.delete -> {
                        // Delete from the database and remove from the RecyclerView
                        mStudyDb.deleteSubject(mSelectedSubject)
                        mSubjectAdapter.removeSubject(mSelectedSubject)

                        // Close the CAB
                        mode.finish()
                        true
                    }
                    else -> false
                }
            }

            override fun onDestroyActionMode(mode: ActionMode) {
                mActionMode = null

                // CAB closing, need to deselect item if not deleted
                mSubjectAdapter.notifyItemChanged(mSelectedSubjectPosition)
                mSelectedSubjectPosition = RecyclerView.NO_POSITION
            }
        }

        init {
            itemView.setOnClickListener(this)
            itemView.setOnLongClickListener(this)
            mTextView = itemView.findViewById(R.id.subjectTextView)
        }
    }

    private inner class SubjectAdapter(private val mSubjectList: MutableList<Subject>) : RecyclerView.Adapter<SubjectHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): SubjectHolder {
            val layoutInflater = LayoutInflater.from(applicationContext)
            return SubjectHolder(layoutInflater, parent)
        }

        override fun onBindViewHolder(holder: SubjectHolder, position: Int) {
            holder.bind(mSubjectList[position], position)
        }

        override fun getItemCount(): Int {
            return mSubjectList.size
        }

        fun onSubjectEntered(subject: String) {
            // Returns subject entered in the SubjectDialogFragment dialog
            if (subject.isNotEmpty()) {
                val sub = Subject(subject)
                if (mStudyDb.addSubject(sub)) {
                    mSubjectAdapter.addSubject(sub)
//                    Toast.makeText(this, subject, Toast.LENGTH_SHORT).show()
                    Toast.makeText(this@SubjectActivity, subject, Toast.LENGTH_LONG).show()
                } else {
                    val message = resources.getString(R.string.subject_exists, subject)
                    Toast.makeText(this@SubjectActivity, message, Toast.LENGTH_SHORT).show()
                }
            }
        }

        fun removeSubject(subject: Subject?) {
            // Find subject in the list
            val index = mSubjectList.indexOf(subject)
            if (index >= 0) {
                // Remove the subject
                mSubjectList.removeAt(index)

                // Notify adapter of subject removal
                notifyItemRemoved(index)
            }
        }

        fun addSubject(subject: Subject?) {
            // Add the new subject at the beginning of the list
            mSubjectList.add(0, subject!!)

            // Notify the adapter that item was added to the beginning of the list
            notifyItemInserted(0)

            // Scroll to the top
            mRecyclerView.scrollToPosition(0)
        }
    }
}